@@ -20,20 +20,23 @@
 from __future__ import annotations
 
 import abc
-import configparser
-from typing import Callable, Dict, List, Optional, Union
+from typing import Callable, Dict, List, Optional, TYPE_CHECKING, Union
 
 import fenics
 import numpy as np
 from typing_extensions import Literal
 import ufl
 
+from cashocs import _exceptions
 from cashocs import _utils
-from cashocs._constraints import constraints
 from cashocs._constraints import solvers
 from cashocs._optimization import optimal_control
 from cashocs._optimization import shape_optimization
 
+if TYPE_CHECKING:
+    from cashocs import io
+    from cashocs import types
+
 
 def _hook() -> None:
     return None
@@ -42,27 +45,30 @@ def _hook() -> None:
 class ConstrainedOptimizationProblem(abc.ABC):
     """An optimization problem with additional equality / inequality constraints."""
 
+    solver: Union[solvers.AugmentedLagrangianMethod, solvers.QuadraticPenaltyMethod]
+    scalar_tracking_forms_initial: Optional[List[Dict]]
+
     def __init__(
         self,
         state_forms: Union[List[ufl.Form], ufl.Form],
         bcs_list: Union[
             List[List[fenics.DirichletBC]], List[fenics.DirichletBC], fenics.DirichletBC
         ],
-        cost_functional_form: Union[List[ufl.Form], ufl.Form],
+        cost_functional_form: Union[
+            List[types.CostFunctional], types.CostFunctional, List[ufl.Form], ufl.Form
+        ],
         states: Union[fenics.Function, List[fenics.Function]],
         adjoints: Union[fenics.Function, List[fenics.Function]],
-        constraint_list: Union[
-            List[
-                Union[constraints.EqualityConstraint, constraints.InequalityConstraint]
-            ],
-            constraints.EqualityConstraint,
-            constraints.InequalityConstraint,
-        ],
-        config: Optional[configparser.ConfigParser] = None,
+        constraint_list: Union[List[types.Constraint], types.Constraint],
+        config: Optional[io.Config] = None,
         initial_guess: Optional[List[fenics.Function]] = None,
-        ksp_options: Optional[List[List[List[str]]]] = None,
-        adjoint_ksp_options: Optional[List[List[List[str]]]] = None,
-        scalar_tracking_forms: Optional[List[Dict]] = None,
+        ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        adjoint_ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        scalar_tracking_forms: Optional[Union[Dict, List[Dict]]] = None,
     ) -> None:
         """Initializes self.
 
@@ -107,13 +113,15 @@ class ConstrainedOptimizationProblem(abc.ABC):
         self.bcs_list = bcs_list
         self.states = states
         self.adjoints = adjoints
-        self.config = config
+        if config is None:
+            self.config = io.Config()
+        else:
+            self.config = config
         self.initial_guess = initial_guess
         self.ksp_options = ksp_options
         self.adjoint_ksp_options = adjoint_ksp_options
 
-        self.solver = None
-        self.current_function_value = None
+        self.current_function_value = 0.0
 
         self._pre_hook = _hook
         self._post_hook = _hook
@@ -123,7 +131,7 @@ class ConstrainedOptimizationProblem(abc.ABC):
             self.scalar_tracking_forms_initial = _utils.enlist(scalar_tracking_forms)
         else:
             self.scalar_tracking_forms_initial = None
-        self.constraint_list = _utils.enlist(constraint_list)
+        self.constraint_list: List[types.Constraint] = _utils.enlist(constraint_list)
 
         self.constraint_dim = len(self.constraint_list)
 
@@ -197,7 +205,8 @@ class ConstrainedOptimizationProblem(abc.ABC):
         for constraint in self.constraint_list:
             s += pow(constraint.constraint_violation(), 2)
 
-        return np.sqrt(s)
+        violation: float = np.sqrt(s)
+        return violation
 
     @abc.abstractmethod
     def _solve_inner_problem(
@@ -265,24 +274,31 @@ class ConstrainedOptimalControlProblem(ConstrainedOptimizationProblem):
         bcs_list: Union[
             fenics.DirichletBC, List[fenics.DirichletBC], List[List[fenics.DirichletBC]]
         ],
-        cost_functional_form: Union[ufl.Form, List[ufl.Form]],
+        cost_functional_form: Union[
+            List[types.CostFunctional], types.CostFunctional, List[ufl.Form], ufl.Form
+        ],
         states: Union[fenics.Function, List[fenics.Function]],
         controls: Union[fenics.Function, List[fenics.Function]],
         adjoints: Union[fenics.Function, List[fenics.Function]],
-        constraint_list: Union[
-            constraints.EqualityConstraint,
-            constraints.InequalityConstraint,
-            List[
-                Union[constraints.EqualityConstraint, constraints.InequalityConstraint]
-            ],
-        ],
-        config: Optional[configparser.ConfigParser] = None,
+        constraint_list: Union[types.Constraint, List[types.Constraint]],
+        config: Optional[io.Config] = None,
         riesz_scalar_products: Optional[Union[ufl.Form, List[ufl.Form]]] = None,
         control_constraints: Optional[List[List[Union[float, fenics.Function]]]] = None,
         initial_guess: Optional[List[fenics.Function]] = None,
-        ksp_options: Optional[List[List[List[str]]]] = None,
-        adjoint_ksp_options: Optional[List[List[List[str]]]] = None,
+        ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        adjoint_ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
         scalar_tracking_forms: Optional[Union[Dict, List[Dict]]] = None,
+        control_bcs_list: Optional[
+            Union[
+                fenics.DirichletBC,
+                List[fenics.DirichletBC],
+                List[List[fenics.DirichletBC]],
+            ]
+        ] = None,
     ) -> None:
         r"""Initializes self.
 
@@ -330,6 +346,8 @@ class ConstrainedOptimalControlProblem(ConstrainedOptimizationProblem):
                 desired value. Each dict needs to have the keys ``'integrand'`` and
                 ``'tracking_goal'``. Default is ``None``, i.e., no scalar tracking terms
                 are considered.
+            control_bcs_list: A list of boundary conditions for the control variables.
+                This is passed analogously to ``bcs_list``. Default is ``None``.
 
         """
         super().__init__(
@@ -348,6 +366,7 @@ class ConstrainedOptimalControlProblem(ConstrainedOptimizationProblem):
 
         self.controls = controls
         self.riesz_scalar_products = riesz_scalar_products
+        self.control_bcs_list = control_bcs_list
         self.control_constraints = control_constraints
 
     def _solve_inner_problem(
@@ -383,8 +402,8 @@ class ConstrainedOptimalControlProblem(ConstrainedOptimizationProblem):
             initial_guess=self.initial_guess,
             ksp_options=self.ksp_options,
             adjoint_ksp_options=self.adjoint_ksp_options,
-            scalar_tracking_forms=self.solver.inner_scalar_tracking_forms,
-            min_max_terms=self.solver.inner_min_max_terms,
+            scalar_tracking_forms=self.scalar_tracking_forms_initial,
+            control_bcs_list=self.control_bcs_list,
         )
 
         optimal_control_problem.inject_pre_post_hook(self._pre_hook, self._post_hook)
@@ -432,20 +451,22 @@ class ConstrainedShapeOptimizationProblem(ConstrainedOptimizationProblem):
             List[List[fenics.DirichletBC]],
             None,
         ],
-        cost_functional_form: Union[ufl.Form, List[ufl.Form]],
+        cost_functional_form: Union[
+            List[types.CostFunctional], types.CostFunctional, List[ufl.Form], ufl.Form
+        ],
         states: Union[fenics.Function, List[fenics.Function]],
         adjoints: Union[fenics.Function, List[fenics.Function]],
         boundaries: fenics.MeshFunction,
-        constraint_list: Union[
-            constraints.EqualityConstraint,
-            constraints.InequalityConstraint,
-            List[constraints.EqualityConstraint, constraints.InequalityConstraint],
-        ],
-        config: Optional[configparser.ConfigParser] = None,
+        constraint_list: Union[types.Constraint, List[types.Constraint]],
+        config: Optional[io.Config] = None,
         shape_scalar_product: Optional[ufl.Form] = None,
         initial_guess: Optional[List[fenics.Function]] = None,
-        ksp_options: Optional[List[List[List[str]]]] = None,
-        adjoint_ksp_options: Optional[List[List[List[str]]]] = None,
+        ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        adjoint_ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
         scalar_tracking_forms: Optional[Dict] = None,
     ) -> None:
         """Initializes self.
@@ -509,6 +530,11 @@ class ConstrainedShapeOptimizationProblem(ConstrainedOptimizationProblem):
             scalar_tracking_forms=scalar_tracking_forms,
         )
 
+        if self.config.getboolean("Mesh", "remesh"):
+            raise _exceptions.CashocsException(
+                "Remeshing is not yet supported for constrained problems"
+            )
+
         self.boundaries = boundaries
         self.shape_scalar_product = shape_scalar_product
 
@@ -544,8 +570,7 @@ class ConstrainedShapeOptimizationProblem(ConstrainedOptimizationProblem):
             initial_guess=self.initial_guess,
             ksp_options=self.ksp_options,
             adjoint_ksp_options=self.adjoint_ksp_options,
-            scalar_tracking_forms=self.solver.inner_scalar_tracking_forms,
-            min_max_terms=self.solver.inner_min_max_terms,
+            scalar_tracking_forms=self.scalar_tracking_forms_initial,
         )
         shape_optimization_problem.inject_pre_post_hook(self._pre_hook, self._post_hook)
         shape_optimization_problem.shift_cost_functional(
