@@ -19,12 +19,10 @@
 
 from __future__ import annotations
 
-import configparser
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, TYPE_CHECKING, Union
 
 import fenics
 import numpy as np
-from typing_extensions import Literal
 import ufl
 
 from cashocs import _exceptions
@@ -38,6 +36,10 @@ from cashocs._optimization import optimization_algorithms
 from cashocs._optimization import optimization_problem
 from cashocs._optimization import verification
 
+if TYPE_CHECKING:
+    from cashocs import io
+    from cashocs import types
+
 
 class OptimalControlProblem(optimization_problem.OptimizationProblem):
     """Implements an optimal control problem.
@@ -52,25 +54,41 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
     ``y1`` and so on).
     """
 
+    controls: List[fenics.Function]
+    control_spaces: List[fenics.FunctionSpace]
+
     def __new__(
         cls,
-        state_forms: Union[ufl.Form, List[ufl.Form]],
+        state_forms: Union[List[ufl.Form], ufl.Form],
         bcs_list: Union[
-            fenics.DirichletBC, List[fenics.DirichletBC], List[List[fenics.DirichletBC]]
+            List[List[fenics.DirichletBC]], List[fenics.DirichletBC], fenics.DirichletBC
         ],
-        cost_functional_form: Union[ufl.Form, List[ufl.Form]],
-        states: Union[fenics.Function, List[fenics.Function]],
-        controls: Union[fenics.Function, List[fenics.Function]],
-        adjoints: Union[fenics.Function, List[fenics.Function]],
-        config: Optional[configparser.ConfigParser] = None,
-        riesz_scalar_products: Optional[Union[ufl.Form, List[ufl.Form]]] = None,
+        cost_functional_form: Union[
+            List[types.CostFunctional], types.CostFunctional, List[ufl.Form], ufl.Form
+        ],
+        states: Union[List[fenics.Function], fenics.Function],
+        controls: Union[List[fenics.Function], fenics.Function],
+        adjoints: Union[List[fenics.Function], fenics.Function],
+        config: Optional[io.Config] = None,
+        riesz_scalar_products: Optional[Union[List[ufl.Form], ufl.Form]] = None,
         control_constraints: Optional[List[List[Union[float, fenics.Function]]]] = None,
         initial_guess: Optional[List[fenics.Function]] = None,
-        ksp_options: Optional[List[List[List[str]]]] = None,
-        adjoint_ksp_options: Optional[List[List[List[str]]]] = None,
-        scalar_tracking_forms: Optional[Union[Dict, List[Dict]]] = None,
-        min_max_terms: Optional[List[Dict]] = None,
+        ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        adjoint_ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        scalar_tracking_forms: Optional[Union[List[Dict], Dict]] = None,
+        min_max_terms: Optional[Union[List[Dict], Dict]] = None,
         desired_weights: Optional[List[float]] = None,
+        control_bcs_list: Optional[
+            Union[
+                List[List[fenics.DirichletBC]],
+                List[fenics.DirichletBC],
+                fenics.DirichletBC,
+            ]
+        ] = None,
     ) -> OptimalControlProblem:
         r"""Initializes self.
 
@@ -124,6 +142,8 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
                 magnitude of `desired_weights[i]` for the initial iteration. In case
                 that `desired_weights` is `None`, no scaling is performed. Default is
                 `None`.
+            control_bcs_list: A list of boundary conditions for the control variables.
+                This is passed analogously to ``bcs_list``. Default is ``None``.
 
         Examples:
             Examples how to use this class can be found in the :ref:`tutorial
@@ -134,7 +154,7 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
 
         if use_scaling:
             unscaled_problem = super().__new__(cls)
-            unscaled_problem.__init__(
+            unscaled_problem.__init__(  # type: ignore
                 state_forms,
                 bcs_list,
                 cost_functional_form,
@@ -150,6 +170,7 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
                 scalar_tracking_forms=scalar_tracking_forms,
                 min_max_terms=min_max_terms,
                 desired_weights=desired_weights,
+                control_bcs_list=control_bcs_list,
             )
             unscaled_problem._scale_cost_functional()  # overwrites cost functional list
 
@@ -157,23 +178,36 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
 
     def __init__(
         self,
-        state_forms: Union[ufl.Form, List[ufl.Form]],
+        state_forms: Union[List[ufl.Form], ufl.Form],
         bcs_list: Union[
-            fenics.DirichletBC, List[fenics.DirichletBC], List[List[fenics.DirichletBC]]
+            List[List[fenics.DirichletBC]], List[fenics.DirichletBC], fenics.DirichletBC
         ],
-        cost_functional_form: Union[ufl.Form, List[ufl.Form]],
-        states: Union[fenics.Function, List[fenics.Function]],
-        controls: Union[fenics.Function, List[fenics.Function]],
-        adjoints: Union[fenics.Function, List[fenics.Function]],
-        config: Optional[configparser.ConfigParser] = None,
-        riesz_scalar_products: Optional[Union[ufl.Form, List[ufl.Form]]] = None,
+        cost_functional_form: Union[
+            List[types.CostFunctional], types.CostFunctional, List[ufl.Form], ufl.Form
+        ],
+        states: Union[List[fenics.Function], fenics.Function],
+        controls: Union[List[fenics.Function], fenics.Function],
+        adjoints: Union[List[fenics.Function], fenics.Function],
+        config: Optional[io.Config] = None,
+        riesz_scalar_products: Optional[Union[List[ufl.Form], ufl.Form]] = None,
         control_constraints: Optional[List[List[Union[float, fenics.Function]]]] = None,
         initial_guess: Optional[List[fenics.Function]] = None,
-        ksp_options: Optional[List[List[List[str]]]] = None,
-        adjoint_ksp_options: Optional[List[List[List[str]]]] = None,
-        scalar_tracking_forms: Optional[Union[Dict, List[Dict]]] = None,
-        min_max_terms: Optional[List[Dict]] = None,
+        ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        adjoint_ksp_options: Optional[
+            Union[types.KspOptions, List[List[Union[str, int, float]]]]
+        ] = None,
+        scalar_tracking_forms: Optional[Union[List[Dict], Dict]] = None,
+        min_max_terms: Optional[Union[List[Dict], Dict]] = None,
         desired_weights: Optional[List[float]] = None,
+        control_bcs_list: Optional[
+            Union[
+                List[List[fenics.DirichletBC]],
+                List[fenics.DirichletBC],
+                fenics.DirichletBC,
+            ]
+        ] = None,
     ) -> None:
         r"""Initializes self.
 
@@ -227,6 +261,8 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
                 magnitude of `desired_weights[i]` for the initial iteration. In case
                 that `desired_weights` is `None`, no scaling is performed. Default is
                 `None`.
+            control_bcs_list: A list of boundary conditions for the control variables.
+                This is passed analogously to ``bcs_list``. Default is ``None``.
 
         Examples:
             Examples how to use this class can be found in the :ref:`tutorial
@@ -256,6 +292,25 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
             riesz_scalar_products
         )
 
+        self.use_control_bcs = False
+        self.control_bcs_list: Union[List[List[fenics.DirichletBC]], List[None]]
+        if control_bcs_list is not None:
+            self.control_bcs_list_inhomogeneous = _utils.check_and_enlist_bcs(
+                control_bcs_list
+            )
+            self.control_bcs_list = []  # type: ignore
+            for list_bcs in self.control_bcs_list_inhomogeneous:
+                hom_bcs: List[fenics.DirichletBC] = [
+                    fenics.DirichletBC(bc) for bc in list_bcs
+                ]
+                for bc in hom_bcs:
+                    bc.homogenize()
+                self.control_bcs_list.append(hom_bcs)  # type: ignore
+
+            self.use_control_bcs = True
+        else:
+            self.control_bcs_list = [None] * self.control_dim
+
         # control_constraints
         self.control_constraints = self._parse_control_constraints(control_constraints)
         self._validate_control_constraints()
@@ -263,15 +318,13 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
         # end overloading
 
         self.is_control_problem = True
-        self.form_handler = _forms.ControlFormHandler(self)
+        self.form_handler: _forms.ControlFormHandler = _forms.ControlFormHandler(self)
 
         self.state_spaces = self.form_handler.state_spaces
         self.control_spaces = self.form_handler.control_spaces
         self.adjoint_spaces = self.form_handler.adjoint_spaces
 
-        self.projected_difference = [
-            fenics.Function(function_space) for function_space in self.control_spaces
-        ]
+        self.projected_difference = _utils.create_function_list(self.control_spaces)
 
         self.state_problem = _pde_problems.StateProblem(
             self.form_handler, self.initial_guess
@@ -279,8 +332,10 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
         self.adjoint_problem = _pde_problems.AdjointProblem(
             self.form_handler, self.state_problem
         )
-        self.gradient_problem = _pde_problems.ControlGradientProblem(
-            self.form_handler, self.state_problem, self.adjoint_problem
+        self.gradient_problem: _pde_problems.ControlGradientProblem = (
+            _pde_problems.ControlGradientProblem(
+                self.form_handler, self.state_problem, self.adjoint_problem
+            )
         )
 
         self.algorithm = _utils.optimization_algorithm_configuration(self.config)
@@ -301,20 +356,16 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
         super()._erase_pde_memory()
         self.gradient_problem.has_solution = False
 
+    def _setup_control_bcs(self) -> None:
+        """Sets up the boundary conditions for the control variables."""
+        if self.use_control_bcs:
+            for i in range(self.control_dim):
+                for bc in self.control_bcs_list_inhomogeneous[i]:
+                    bc.apply(self.controls[i].vector())
+
     def solve(
         self,
-        algorithm: Optional[
-            Literal[
-                "gradient_descent",
-                "gd",
-                "conjugate_gradient",
-                "nonlinear_cg",
-                "ncg",
-                "lbfgs",
-                "bfgs",
-                "newton",
-            ]
-        ] = None,
+        algorithm: Optional[str] = None,
         rtol: Optional[float] = None,
         atol: Optional[float] = None,
         max_iter: Optional[int] = None,
@@ -367,6 +418,8 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
         """
         super().solve(algorithm=algorithm, rtol=rtol, atol=atol, max_iter=max_iter)
 
+        self._setup_control_bcs()
+
         self.optimization_variable_abstractions = (
             optimal_control.ControlVariableAbstractions(self)
         )
@@ -430,11 +483,17 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
             the control variables.
 
         """
-        mod_derivatives = None
-        if isinstance(derivatives, list) and len(derivatives) > 0:
-            mod_derivatives = derivatives
-        elif isinstance(derivatives, ufl.form.Form):
+        mod_derivatives: List[ufl.Form]
+        if isinstance(derivatives, ufl.form.Form):
             mod_derivatives = [derivatives]
+        else:
+            mod_derivatives = derivatives
+
+        self.form_handler.setup_assemblers(
+            self.form_handler.riesz_scalar_products,
+            mod_derivatives,
+            self.form_handler.control_bcs_list,
+        )
 
         self.form_handler.gradient_forms_rhs = mod_derivatives
         self.has_custom_derivative = True
@@ -514,7 +573,8 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
             return _utils.enlist(riesz_scalar_products)
 
     def _parse_control_constraints(
-        self, control_constraints
+        self,
+        control_constraints: Optional[List[List[Union[float, fenics.Function]]]],
     ) -> List[List[fenics.Function]]:
         """Checks, whether the given control constraints are feasible.
 
@@ -525,13 +585,16 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
             The (wrapped) list of control constraints.
 
         """
+        temp_control_constraints: List[List[Union[fenics.Function, float]]]
         if control_constraints is None:
             temp_control_constraints = []
             for control in self.controls:
                 u_a = fenics.Function(control.function_space())
                 u_a.vector().vec().set(float("-inf"))
+                u_a.vector().apply("")
                 u_b = fenics.Function(control.function_space())
                 u_b.vector().vec().set(float("inf"))
+                u_b.vector().apply("")
                 temp_control_constraints.append([u_a, u_b])
         else:
             temp_control_constraints = _utils.check_and_enlist_control_constraints(
@@ -539,21 +602,21 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
             )
 
         # recast floats into functions for compatibility
-        lower_bound = None
-        upper_bound = None
-        formatted_control_constraints = []
+        formatted_control_constraints: List[List[fenics.Function]] = []
         for idx, pair in enumerate(temp_control_constraints):
-            if isinstance(pair[0], (float, int)):
+            if isinstance(pair[0], fenics.Function):
+                lower_bound = pair[0]
+            else:
                 lower_bound = fenics.Function(self.controls[idx].function_space())
                 lower_bound.vector().vec().set(pair[0])
-            elif isinstance(pair[0], fenics.Function):
-                lower_bound = pair[0]
+                lower_bound.vector().apply("")
 
-            if isinstance(pair[1], (float, int)):
+            if isinstance(pair[1], fenics.Function):
+                upper_bound = pair[1]
+            else:
                 upper_bound = fenics.Function(self.controls[idx].function_space())
                 upper_bound.vector().vec().set(pair[1])
-            elif isinstance(pair[1], fenics.Function):
-                upper_bound = pair[1]
+                upper_bound.vector().apply("")
 
             formatted_control_constraints.append([lower_bound, upper_bound])
 
@@ -563,7 +626,7 @@ class OptimalControlProblem(optimization_problem.OptimizationProblem):
         """Checks, whether given control constraints are valid."""
         self.require_control_constraints = [False] * self.control_dim
         for idx, pair in enumerate(self.control_constraints):
-            if not np.alltrue(pair[0].vector()[:] < pair[1].vector()[:]):
+            if not np.all(pair[0].vector()[:] < pair[1].vector()[:]):
                 raise _exceptions.InputError(
                     (
                         "cashocs._optimization.optimal_control."
