@@ -24,11 +24,11 @@ shape gradient with a Riesz projection.
 from __future__ import annotations
 
 import configparser
-from typing import List, TYPE_CHECKING, Union
+import copy
+from typing import List, TYPE_CHECKING
 
 import fenics
 import numpy as np
-from petsc4py import PETSc
 import ufl
 
 from cashocs import _loggers
@@ -71,19 +71,10 @@ class ShapeGradientProblem(pde_problem.PDEProblem):
 
         gradient_tol = self.config.getfloat("OptimizationRoutine", "gradient_tol")
 
-        # Generate the Krylov solver for the shape gradient problem
-        # noinspection PyUnresolvedReferences
-        self.ksp = PETSc.KSP().create()
-
         gradient_method = self.config.get("OptimizationRoutine", "gradient_method")
 
         if gradient_method.casefold() == "direct":
-            self.ksp_options: List[List[Union[str, int, float]]] = [
-                ["ksp_type", "preonly"],
-                ["pc_type", "lu"],
-                ["pc_factor_mat_solver_type", "mumps"],
-                ["mat_mumps_icntl_24", 1],
-            ]
+            self.ksp_options = copy.deepcopy(_utils.linalg.direct_ksp_options)
         elif gradient_method.casefold() == "iterative":
             self.ksp_options = [
                 ["ksp_type", "cg"],
@@ -95,8 +86,6 @@ class ShapeGradientProblem(pde_problem.PDEProblem):
                 ["ksp_max_it", 250],
             ]
 
-        _utils.setup_petsc_options([self.ksp], [self.ksp_options])
-
         if (
             self.config.getboolean("ShapeGradient", "use_p_laplacian")
             and self.form_handler.use_fixed_dimensions
@@ -158,11 +147,10 @@ class ShapeGradientProblem(pde_problem.PDEProblem):
                     )
                     self.form_handler.fe_shape_derivative_vector.apply("")
                 _utils.solve_linear_problem(
-                    self.ksp,
-                    self.form_handler.scalar_product_matrix,
-                    self.form_handler.fe_shape_derivative_vector.vec(),
-                    self.gradient[0].vector().vec(),
-                    self.ksp_options,
+                    A=self.form_handler.scalar_product_matrix,
+                    b=self.form_handler.fe_shape_derivative_vector.vec(),
+                    x=self.gradient[0].vector().vec(),
+                    ksp_options=self.ksp_options,
                 )
                 self.gradient[0].vector().apply("")
 
@@ -234,12 +222,7 @@ class _PLaplaceProjector:
             gradient_method = config.get("OptimizationRoutine", "gradient_method")
 
             if gradient_method.casefold() == "direct":
-                self.ksp_options: List[List[Union[str, int, float]]] = [
-                    ["ksp_type", "preonly"],
-                    ["pc_type", "lu"],
-                    ["pc_factor_mat_solver_type", "mumps"],
-                    ["mat_mumps_icntl_24", 1],
-                ]
+                self.ksp_options = copy.deepcopy(_utils.linalg.direct_ksp_options)
             elif gradient_method.casefold() == "iterative":
                 self.ksp_options = [
                     ["ksp_type", "cg"],
