@@ -19,6 +19,7 @@
 
 from __future__ import annotations
 
+import copy
 from typing import List, Optional, Tuple, TYPE_CHECKING, Union
 
 import fenics
@@ -32,6 +33,23 @@ from cashocs._utils import forms as forms_module
 if TYPE_CHECKING:
     from cashocs import types
 
+iterative_ksp_options: List[List[Union[str, int, float]]] = [
+    ["ksp_type", "cg"],
+    ["pc_type", "hypre"],
+    ["pc_hypre_type", "boomeramg"],
+    ["pc_hypre_boomeramg_strong_threshold", 0.7],
+    ["ksp_rtol", 1e-20],
+    ["ksp_atol", 1e-50],
+    ["ksp_max_it", 1000],
+]
+
+direct_ksp_options: List[List[Union[str, int, float]]] = [
+    ["ksp_type", "preonly"],
+    ["pc_type", "lu"],
+    ["pc_factor_mat_solver_type", "mumps"],
+    ["mat_mumps_icntl_24", 1],
+]
+
 
 def split_linear_forms(forms: List[ufl.Form]) -> Tuple[List[ufl.Form], List[ufl.Form]]:
     """Splits a list of linear forms into left- and right-hand sides.
@@ -78,7 +96,6 @@ def split_linear_forms(forms: List[ufl.Form]) -> Tuple[List[ufl.Form], List[ufl.
     return lhs_list, rhs_list
 
 
-# noinspection PyUnresolvedReferences
 def assemble_petsc_system(
     lhs_form: ufl.Form,
     rhs_form: ufl.Form,
@@ -120,14 +137,12 @@ def assemble_petsc_system(
     )
     A_tensor.ident_zeros()
 
-    # noinspection PyPep8Naming
     A = A_tensor.mat()  # pylint: disable=invalid-name
     b = b_tensor.vec()
 
     return A, b
 
 
-# noinspection PyUnresolvedReferences
 def setup_petsc_options(ksps: List[PETSc.KSP], ksp_options: types.KspOptions) -> None:
     """Sets up an (iterative) linear solver.
 
@@ -144,7 +159,6 @@ def setup_petsc_options(ksps: List[PETSc.KSP], ksp_options: types.KspOptions) ->
     opts = fenics.PETScOptions
 
     for i in range(len(ksps)):
-        # noinspection PyArgumentList
         opts.clear()
 
         for option in ksp_options[i]:
@@ -153,9 +167,7 @@ def setup_petsc_options(ksps: List[PETSc.KSP], ksp_options: types.KspOptions) ->
         ksps[i].setFromOptions()
 
 
-# noinspection PyUnresolvedReferences
 def solve_linear_problem(
-    ksp: Optional[PETSc.KSP] = None,
     A: Optional[PETSc.Mat] = None,  # pylint: disable=invalid-name
     b: Optional[PETSc.Vec] = None,
     x: Optional[PETSc.Vec] = None,
@@ -166,8 +178,6 @@ def solve_linear_problem(
     """Solves a finite dimensional linear problem.
 
     Args:
-        ksp: The PETSc KSP object used to solve the problem. None means that the solver
-            mumps is used (default is None).
         A: The PETSc matrix corresponding to the left-hand side of the problem. If
             this is None, then the matrix stored in the ksp object is used. Raises
             an error if no matrix is stored. Default is None.
@@ -189,21 +199,11 @@ def solve_linear_problem(
         The solution vector.
 
     """
-    if ksp is None:
-        ksp = PETSc.KSP().create()
-        options: List[List[Union[str, int, float]]] = [
-            ["ksp_type", "preonly"],
-            ["pc_type", "lu"],
-            ["pc_factor_mat_solver_type", "mumps"],
-            ["mat_mumps_icntl_24", 1],
-        ]
-
-        setup_petsc_options([ksp], [options])
+    ksp = PETSc.KSP().create()
 
     if A is not None:
         ksp.setOperators(A)
     else:
-        # noinspection PyPep8Naming
         A = ksp.getOperators()[0]
         if A.size[0] == -1 and A.size[1] == -1:
             raise _exceptions.InputError(
@@ -219,8 +219,12 @@ def solve_linear_problem(
     if x is None:
         x, _ = A.getVecs()
 
-    if ksp_options is not None:
-        setup_petsc_options([ksp], [ksp_options])
+    if ksp_options is None:
+        options = copy.deepcopy(direct_ksp_options)
+    else:
+        options = ksp_options
+
+    setup_petsc_options([ksp], [options])
 
     if rtol is not None:
         ksp.rtol = rtol
@@ -234,7 +238,6 @@ def solve_linear_problem(
     return x
 
 
-# noinspection PyUnresolvedReferences
 def assemble_and_solve_linear(
     lhs_form: ufl.Form,
     rhs_form: ufl.Form,
@@ -242,7 +245,6 @@ def assemble_and_solve_linear(
     A: Optional[fenics.PETScMatrix] = None,  # pylint: disable=invalid-name
     b: Optional[fenics.PETScVector] = None,
     x: Optional[PETSc.Vec] = None,
-    ksp: Optional[PETSc.KSP] = None,
     ksp_options: Optional[List[List[Union[str, int, float]]]] = None,
     rtol: Optional[float] = None,
     atol: Optional[float] = None,
@@ -257,8 +259,6 @@ def assemble_and_solve_linear(
         b: A vector into which the rhs is assembled. Default is ``None``.
         x: The PETSc vector that stores the solution of the problem. If this is
             None, then a new vector will be created (and returned).
-        ksp: The PETSc KSP object used to solve the problem. None means that the solver
-            mumps is used (default is None).
         ksp_options: The options for the PETSc ksp object. If this is None (the default)
             a direct method is used.
         rtol: The relative tolerance used in case an iterative solver is used for
@@ -273,12 +273,10 @@ def assemble_and_solve_linear(
 
     """
     # pylint: disable=invalid-name
-    # noinspection PyPep8Naming
     A_matrix, b_vector = assemble_petsc_system(
         lhs_form, rhs_form, bcs, A_tensor=A, b_tensor=b
     )
     solution = solve_linear_problem(
-        ksp=ksp,
         A=A_matrix,
         b=b_vector,
         x=x,
@@ -356,7 +354,6 @@ class Interpolator:
 
         self.origin_space = origin_space
         self.target_space = target_space
-        # noinspection PyTypeChecker
         self.transfer_matrix = fenics.PETScDMCollection.create_transfer_matrix(
             self.origin_space, self.target_space
         ).mat()
@@ -376,14 +373,11 @@ class Interpolator:
             The result of the interpolation.
 
         """
-        if fenics.MPI.comm_world.size <= 1:
-            v = fenics.Function(self.target_space)
-            x = fenics.as_backend_type(u.vector()).vec()
-            _, temp = self.transfer_matrix.getVecs()
-            self.transfer_matrix.mult(x, temp)
-            v.vector().vec().aypx(0.0, temp)
-            v.vector().apply("")
-        else:
-            v = fenics.interpolate(u, self.target_space)
+        v = fenics.Function(self.target_space)
+        x = fenics.as_backend_type(u.vector()).vec()
+        _, temp = self.transfer_matrix.getVecs()
+        self.transfer_matrix.mult(x, temp)
+        v.vector().vec().aypx(0.0, temp)
+        v.vector().apply("")
 
         return v
