@@ -19,11 +19,11 @@
 
 from __future__ import annotations
 
+import copy
 from typing import List, Optional, Union
 
 import fenics
 import numpy as np
-from petsc4py import PETSc
 
 from cashocs import _utils
 from cashocs.geometry import measure
@@ -65,18 +65,7 @@ def compute_boundary_distance(
     function_space = fenics.FunctionSpace(mesh, "CG", 1)
     dx = measure.NamedMeasure("dx", mesh)
 
-    # noinspection PyUnresolvedReferences
-    ksp = PETSc.KSP().create()
-    ksp_options: List[List[Union[str, int, float]]] = [
-        ["ksp_type", "cg"],
-        ["pc_type", "hypre"],
-        ["pc_hypre_type", "boomeramg"],
-        ["pc_hypre_boomeramg_strong_threshold", 0.7],
-        ["ksp_rtol", 1e-20],
-        ["ksp_atol", 1e-50],
-        ["ksp_max_it", 1000],
-    ]
-    _utils.setup_petsc_options([ksp], [ksp_options])
+    ksp_options = copy.deepcopy(_utils.linalg.iterative_ksp_options)
 
     u = fenics.TrialFunction(function_space)
     v = fenics.TestFunction(function_space)
@@ -106,7 +95,9 @@ def compute_boundary_distance(
     lhs = fenics.dot(fenics.grad(u), fenics.grad(v)) * dx
     rhs = fenics.Constant(1.0) * v * dx
 
-    _utils.assemble_and_solve_linear(lhs, rhs, bcs, x=u_curr.vector().vec(), ksp=ksp)
+    _utils.assemble_and_solve_linear(
+        lhs, rhs, bcs, x=u_curr.vector().vec(), ksp_options=ksp_options
+    )
     u_curr.vector().apply("")
 
     rhs = fenics.dot(fenics.grad(u_prev) / norm_u_prev, fenics.grad(v)) * dx
@@ -126,7 +117,7 @@ def compute_boundary_distance(
         u_prev.vector().vec().aypx(0.0, u_curr.vector().vec())
         u_prev.vector().apply("")
         _utils.assemble_and_solve_linear(
-            lhs, rhs, bcs, x=u_curr.vector().vec(), ksp=ksp
+            lhs, rhs, bcs, x=u_curr.vector().vec(), ksp_options=ksp_options
         )
         u_curr.vector().apply("")
         res = np.sqrt(fenics.assemble(residual_form))
